@@ -12,7 +12,7 @@ package MooX::Options::Role;
 use strict;
 use warnings;
 
-our $VERSION = '3.86';    # VERSION
+our $VERSION = '3.88';    # VERSION
 
 use MRO::Compat;
 use Moo::Role;
@@ -26,10 +26,16 @@ use Carp;
 requires qw/_options_data _options_config/;
 
 sub new_with_options {
-    my ( $class, @params ) = @_;
+    my ( $class, %params ) = @_;
     my $self;
+    my %cmdline_params = $class->parse_options(%params);
+
+    if ( $cmdline_params{help} ) {
+        return $class->options_usage( $params{help}, $cmdline_params{help} );
+    }
+
     return $self
-        if eval { $self = $class->new( $class->parse_options(@params) ); 1 };
+        if eval { $self = $class->new(%cmdline_params); 1 };
     if ( $@ =~ /^Attribute\s\((.*?)\)\sis\srequired/x ) {
         print "$1 is missing\n";
     }
@@ -38,10 +44,14 @@ sub new_with_options {
         print join( "\n", ( map { $_ . " is missing" } @missing_required ),
             '' );
     }
+    elsif ( $@ =~ /^(.*?)\srequired/x ) {
+        print "$1 is missing\n";
+    }
     else {
         croak $@;
     }
-    return $class->parse_options( help => 1 );
+    %cmdline_params = $class->parse_options( help => 1 );
+    return $class->options_usage( 1, $cmdline_params{help} );
 }
 
 ## no critic qw/Modules::ProhibitExcessMainComplexity/
@@ -150,12 +160,6 @@ sub parse_options {
     my ( $opt, $usage )
         = describe_options( ("USAGE: %c %o"), @options,
         [ 'help|h', "show this help message" ], @flavour );
-    if ( $opt->help() || defined $params{help} ) {
-        print $usage, "\n";
-        my $exit_code = 0;
-        $exit_code = 0 + $params{help} if defined $params{help};
-        exit($exit_code);
-    }
 
     my %cmdline_params = %params;
     for my $name ( keys %options_data ) {
@@ -175,16 +179,32 @@ sub parse_options {
         }
     }
 
+    if ( $opt->help() || defined $params{help} ) {
+        $cmdline_params{help} = $usage;
+    }
+
     return %cmdline_params;
 }
 ## use critic
 
 sub options_usage {
-    my ( $self, $code, @messages ) = @_;
+    my ( $class, $code, @messages ) = @_;
+    my $usage;
+    if ( @messages
+        && ref $messages[-1] eq 'Getopt::Long::Descriptive::Usage' )
+    {
+        $usage = shift @messages;
+    }
     $code = 0 if !defined $code;
     print join( "\n", @messages, '' ) if @messages;
-    local @ARGV = ();
-    return $self->parse_options( help => $code );
+    if ( !$usage ) {
+        local @ARGV = ();
+        my %cmdline_params = $class->parse_options( help => $code );
+        $usage = $cmdline_params{help};
+    }
+    print $usage . "\n";
+    exit($code) if $code >= 0;
+    return;
 }
 
 1;
@@ -199,7 +219,7 @@ MooX::Options::Role - role that is apply to your object
 
 =head1 VERSION
 
-version 3.86
+version 3.88
 
 =head1 METHODS
 
